/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

typedef enum {
  SCENE_TYPE_INITIAL,
  SCENE_TYPE_OVERWORLD,
} scenetype_t;

#define SCENE_TYPE_COUNT 2

typedef struct {
  scenetype_t current;
  scenetype_t next;
} scene_t;

typedef struct {
  void (*onInit)();
  void (*onActive)();
  void (*onInactive)();
  void (*onUpdate)();
  void (*onRender)();
} scenetypecallback_t;

extern scene_t SCENE;

/**
 * Initializes the scene system.
 */
void sceneInit();

/**
 * Updates the scene system.
 */
void sceneUpdate();

/**
 * Renders the scene system.
 * 
 * Scene rendering is really just an abstraction meant to simplify things for
 * the render host. It is not REQUIRED to be called at all and is not actually
 * implemented in the dusk dir itself, it is overriden somewhere within the
 * render host.
 */
void sceneRender();

/**
 * Sets the current scene. This will happen at the start of the next tick.
 * 
 * @param scene The scene to set.
 */
void sceneSet(const scenetype_t scene);