/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"
#include "tilesetdefs.h"

typedef enum {
  TILESET_NULL,
  TILESET_FONT
} tilesetid_t;

#define TILESET_COUNT 2

typedef struct {
  uint8_t columns;
  uint8_t rows;
} tileset_t;

extern tileset_t TILESETS[];
extern tilesetid_t TILESET_SLOTS[];

/**
 * Binds a tileset to a slot. The host can also use this to say load a texture,
 * or upload to VRAM.
 * 
 * This does not guarantee anything on the game side, rendering is responsible
 * for handling anything visual.
 * 
 * @param id The tileset to bind.
 * @param slot The slot to bind the tileset to.
 */
void tilesetBind(const tilesetid_t id, const uint8_t slot);

/**
 * Gets the slot that a tileset is bound to, or 0xFF if not bound.
 * 
 * @param id The tileset to check.
 * @return The slot the tileset is bound to, or 0xFF if not bound.
 */
uint8_t tilesetGetSlot(const tilesetid_t id);

/**
 * Checks if a tileset is bound to a slot.
 * 
 * @param id The tileset to check.
 * @return TRUE if the tileset is bound, FALSE otherwise.
 */
bool_t tilesetIsBound(const tilesetid_t id);