/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "dusk.h"

typedef uint16_t inputstate_t;

#define INPUT_MENU (1 << 0)
#define INPUT_UP (1 << 1)
#define INPUT_DOWN (1 << 2)
#define INPUT_LEFT (1 << 3)
#define INPUT_RIGHT (1 << 4)
#define INPUT_ACCEPT (1 << 5)
#define INPUT_BACK (1 << 6)

extern inputstate_t INPUT_CURRENT;
extern inputstate_t INPUT_LAST_FRAME;

/**
 * Initializes the input system
 */
void inputInit();

/**
 * Updates the input system for this tick.
 */
void inputUpdate();

/**
 * Requests the platform let us know the current state of each input.
 * 
 * @return Current input state.
 */
inputstate_t inputPlatformState();

/**
 * Returns true if the input was pressed this frame, but not last frame.
 * 
 * @param state Inputs to check, typically a single input.
 * @return True if input was pressed this frame but not last frame.
 */
bool_t inputWasPressed(const inputstate_t state);

/**
 * Returns true if the input was released this frame, but pressed last frame.
 * 
 * @param state Inputs to check, typically a single input.
 * @return True if input was released this frame but was pressed last frame.
 */
bool_t inputWasReleased(const inputstate_t state);

/**
 * Returns true if the input is currently pressed.
 * 
 * @param state Inputs to check, typically a single input.
 * @return True if input is currently pressed.
 */
bool_t inputIsDown(const inputstate_t state);

/**
 * Returns true if the input is currently released.
 * 
 * @param state Inputs to check, typically a single input.
 * @return True if input is currently released.
 */
bool_t inputIsUp(const inputstate_t state);