/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "facing.h"
#include "assert/assert.h"

void facingDirGetRelative(const facingdir_t dir, int8_t *x, int8_t *y) {
  assertNotNull(x, "X cannot be NULL");
  assertNotNull(y, "Y cannot be NULL");

  switch(dir) {
    case FACING_DIRECTION_SOUTH:
      *x = 0;
      *y = -1;
      break;

    case FACING_DIRECTION_EAST:
      *x = 1;
      *y = 0;
      break;

    case FACING_DIRECTION_NORTH:
      *x = 0;
      *y = 1;
      break;

    case FACING_DIRECTION_WEST:
      *x = -1;
      *y = 0;
      break;

    default:
      assertUnreachable("Invalid facing direction.");
  }
}

void facingDirAdd(const facingdir_t dir, uint8_t *x, uint8_t *y) {
  assertNotNull(x, "X cannot be NULL");
  assertNotNull(y, "Y cannot be NULL");

  int8_t dx, dy;
  facingDirGetRelative(dir, &dx, &dy);
  
  *x += dx;
  *y += dy;
}

facingdir_t facingDirFace(
  const uint8_t x1, const uint8_t y1,
  const uint8_t x2, const uint8_t y2
) {
  if(x1 == x2) {
    if(y1 < y2) return FACING_DIRECTION_SOUTH;
    if(y1 > y2) return FACING_DIRECTION_NORTH;
  } else if(y1 == y2) {
    if(x1 < x2) return FACING_DIRECTION_WEST;
    if(x1 > x2) return FACING_DIRECTION_EAST;
  }

  return FACING_DIRECTION_SOUTH;
}