/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "map.h"
#include "assert/assert.h"
#include "util/memory.h"

void mapInit(map_t *map) {
  memoryZero(map, sizeof(map_t));

  // Test
  map->width = 16;
  map->height = 16;
  map->layerCount = 1;
  memorySet(&map->tileIds, 0x01, sizeof(map->tileIds));

  // Test size
  assertTrue(
    (map->width * map->height * map->layerCount) <= OVERWORLD_TILE_COUNT_MAX,
    "Map size exceeds tile count."
  );
}

uint32_t mapGetTileIndex(
  const map_t *map,
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
) {
  assertNotNull(map, "Map cannot be NULL");
  assertTrue(layer < map->layerCount, "Invalid layer");
  assertTrue(x < map->width, "Invalid x");
  assertTrue(y < map->height, "Invalid y");

  return (layer * map->width * map->height) + (y * map->width) + x;
}

tileid_t mapGetTileId(
  const map_t *map,
  const uint32_t tileIndex
) {
  assertNotNull(map, "Map cannot be NULL");
  assertMapIndexValid(map, tileIndex);

  return map->tileIds[tileIndex];
}

tileid_t mapGetTileIdAtPosition(
  const map_t *map,
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
) {
  return mapGetTileId(map, mapGetTileIndex(map, layer, x, y));
}

tiledata_t mapGetTileData(
  const map_t *map,
  const uint32_t tileIndex
) {
  assertNotNull(map, "Map cannot be NULL");
  assertMapIndexValid(map, tileIndex);

  return map->tileData[tileIndex];
}