/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "tile.h"
#include "overworld/overworlddefs.h"
#include "assert/assert.h"

#define assertMapIndexValid(map, index) \
  assertTrue( \
    index < (map->layerCount * map->width * map->height), \
    "Invalid tile index" \
  );

typedef struct {
  tileid_t tileIds[OVERWORLD_TILE_COUNT_MAX];
  tiledata_t tileData[OVERWORLD_TILE_COUNT_MAX];
  uint8_t width, height, layerCount;
} map_t;

void mapInit(map_t *map);

/**
 * Returns the index of the tile at the given position.
 * 
 * @param map The map to get the tile index from.
 * @param layer The layer to get the tile from.
 * @param x The x position of the tile.
 * @param y The y position of the tile.
 * @return The index of the tile at the given position.
 */
uint32_t mapGetTileIndex(
  const map_t *map,
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
);

/**
 * Returns the tile ID at the given index.
 * 
 * @param map The map to get the tile ID from.
 * @param tileIndex The index of the tile to get.
 * @return The tile ID at the given index.
 */
tileid_t mapGetTileId(
  const map_t *map,
  const uint32_t tileIndex
);

/**
 * Returns the tile ID at the given position. Shorthand for both getting the 
 * map index and the tile ID.
 * 
 * @param map The map to get the tile ID from.
 * @param layer The layer to get the tile from.
 * @param x The x position of the tile.
 * @param y The y position of the tile.
 * @return The tile ID at the given position.
 */
tileid_t mapGetTileIdAtPosition(
  const map_t *map,
  const uint8_t layer,
  const uint8_t x,
  const uint8_t y
);

/**
 * Returns the tile data at the given index.
 * 
 * @param map The map to get the tile data from.
 * @param tileIndex The index of the tile to get.
 * @return The tile data at the given index.
 */
tiledata_t mapGetTileData(
  const map_t *map,
  const uint32_t tileIndex
);