/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "overworld.h"
#include "util/memory.h"
#include "assert/assert.h"
#include "display/tileset.h"

overworld_t OVERWORLD;

void overworldInit() {
  memoryZero(&OVERWORLD, sizeof(overworld_t));

  mapInit(&OVERWORLD.map);
  
  entityInit(OVERWORLD.entities + OVERWORLD.entityCount++, ENTITY_TYPE_PLAYER);
  
  entityInit(OVERWORLD.entities + OVERWORLD.entityCount++, ENTITY_TYPE_NPC);
  OVERWORLD.entities[1].x = 2;
  OVERWORLD.entities[1].y = 2;
}

void overworldSceneInit() {
  tilesetBind(TILESET_FONT, TILESET_SLOT_ENTITIES);
}

void overworldSceneDeinit() {

}

void overworldUpdate() {
  uint8_t i = 0;
  while(i < OVERWORLD.entityCount) {
    entityUpdate(OVERWORLD.entities + i++);
  }
}

entity_t * overworldEntityGetAtPosition(const uint8_t x, const uint8_t y) {
  uint8_t i = 0;
  while(i < OVERWORLD.entityCount) {
    entity_t * entity = OVERWORLD.entities + i++;
    if(entity->x == x && entity->y == y) return entity;
  }
  return NULL;
}