/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "overworlddefs.h"
#include "overworld/entity/entity.h"
#include "overworld/map/map.h"

typedef struct {
  entity_t entities[OVERWORLD_ENTITY_COUNT_MAX];
  uint8_t entityCount;
  map_t map;
} overworld_t;

extern overworld_t OVERWORLD;

/**
 * Initializes the overworld.
 */
void overworldInit();

/**
 * Called by the scene system when the overworld scene is made to be the 
 * active scene.
 */
void overworldSceneInit();

/**
 * Called by the scene system when the overworld scene is made to be inactive.
 */
void overworldSceneDeinit();

/**
 * Called by the scene system when the overworld scene is to perform a tick.
 */
void overworldUpdate();

/**
 * Platform level render method. Refer to sceneRender for information.
 */
void overworldRender();

/**
 * Returns the entity at the given position.
 * 
 * @param x The x position of the entity.
 * @param y The y position of the entity.
 * @return The entity at the given position.
 */
entity_t * overworldEntityGetAtPosition(
  const uint8_t x,
  const uint8_t y
);