/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "duskgl.h"

typedef struct {
  GLuint id;
  GLuint texture;
  int width;
  int height;
} framebuffer_t;

extern framebuffer_t FRAMEBUFFER;

/**
 * Initializes the framebuffer.
 * 
 * @param width The width of the framebuffer.
 * @param height The height of the framebuffer.
 */
void frameBufferInit(const int32_t width, const int32_t height);

/**
 * Sets the size of the framebuffer.
 * 
 * @param width The width of the framebuffer.
 * @param height The height of the framebuffer.
 */
void frameBufferSetSize(const int32_t width, const int32_t height);

/**
 * Binds the framebuffer.
 */
void frameBufferBind();

/**
 * Unbinds the framebuffer.
 */
void frameBufferUnbind();

/**
 * Binds the framebuffer texture to a slot.
 * 
 * @param slot The slot to bind the texture to.
 */
void frameBufferTextureBind(const GLuint slot);

/**
 * Unbinds the framebuffer.
 */
void frameBufferDispose();