/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "assert/assertgl.h"
#include "quad.h"

duskquad_t QUAD;

void quadInit() {
  // Create the single quad.
  const float quadPositions[] = {
    0.0f, 0.0f, 0.0f,  // Bottom-left corner
    1.0f, 0.0f, 0.0f,  // Bottom-right corner
    1.0f, 1.0f, 0.0f,  // Top-right corner
    0.0f, 1.0f, 0.0f   // Top-left corner
  };

  const float quadUVs[] = {
    0.0f, 0.0f,
    1.0f, 0.0f,
    1.0f, 1.0f,
    0.0f, 1.0f
  };

  const uint32_t quadIndices[] = {
    0, 1, 2,
    2, 3, 0,
  };

  glGenVertexArrays(1, &QUAD.quadVAO);
  assertNoGLError();

  glBindVertexArray(QUAD.quadVAO);
  assertNoGLError();

  // Positions
  glGenBuffers(1, &QUAD.quadVBO);
  assertNoGLError();
  
  glBindBuffer(GL_ARRAY_BUFFER, QUAD.quadVBO);
  assertNoGLError();
    
  glBufferData(
    GL_ARRAY_BUFFER, sizeof(quadPositions), quadPositions, GL_STATIC_DRAW
  );
  assertNoGLError();
  
  glVertexAttribPointer(0, 3, GL_FLOAT, GL_FALSE, 3 * sizeof(float), (void*)0);
  assertNoGLError();

  glEnableVertexAttribArray(0);
  assertNoGLError();
  
  // UVs
  glGenBuffers(1, &QUAD.quadUVVBO);
  assertNoGLError();

  glBindBuffer(GL_ARRAY_BUFFER, QUAD.quadUVVBO);
  assertNoGLError();

  glBufferData(GL_ARRAY_BUFFER, sizeof(quadUVs), quadUVs, GL_STATIC_DRAW);
  assertNoGLError();

  glVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 2 * sizeof(float), (void*)0);
  assertNoGLError();

  glEnableVertexAttribArray(1);
  assertNoGLError();

  // Indices
  glGenBuffers(1, &QUAD.quadEBO);
  assertNoGLError();

  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, QUAD.quadEBO);
  assertNoGLError();

  glBufferData(
    GL_ELEMENT_ARRAY_BUFFER, sizeof(quadIndices), quadIndices, GL_STATIC_DRAW
  );
  assertNoGLError();
}

void quadRender(const int32_t count) {
  glBindVertexArray(QUAD.quadVAO);
  assertNoGLError();

  glDrawElementsInstanced(
    GL_TRIANGLES, 6,
    GL_UNSIGNED_INT, 0,
    count
  );
  assertNoGLError();
}

void quadDispose() {
  glDeleteBuffers(1, &QUAD.quadVBO);
  assertNoGLError();

  glDeleteBuffers(1, &QUAD.quadEBO);
  assertNoGLError();

  glDeleteVertexArrays(1, &QUAD.quadVAO);
  assertNoGLError();
}