// Copyright (c) 2025 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "../../../../dusk/overworld/overworlddefs.h"
#include "../fragments/packed.glsl"
#include "../fragments/quad.glsl"

#define MAP_TILE_PACKED_SIZE 16

layout(std140) uniform b_Map {
  uvec4 mapTileIds[OVERWORLD_TILE_COUNT_MAX / MAP_TILE_PACKED_SIZE];
  uvec4 mapSize;
};

vec2 mapTileGetSize() {
  return vec2(float(OVERWORLD_TILE_WIDTH), float(OVERWORLD_TILE_HEIGHT));
}

vec2 mapGetVertice(uint instanceIndex, uint indiceIndex) {
  vec2 quad = quadGetVertice(indiceIndex);

  uint mapWidth = packedGetU8(0u, mapSize);
  uint mapHeight = packedGetU8(1u, mapSize);
  uint mapLayerCount = packedGetU8(2u, mapSize);

  // Get x and y within layer
  uint x = instanceIndex % mapWidth;
  uint y = (instanceIndex / mapWidth) % mapHeight;

  // Get quad position, consider layer count
  quad += vec2(x, y);
  quad *= mapTileGetSize();

  return quad;
}

uint mapGetLayer(uint instanceIndex) {
  uint mapWidth = packedGetU8(0u, mapSize);
  uint mapHeight = packedGetU8(1u, mapSize);
  return instanceIndex / (mapWidth * mapHeight);
}

uint mapGetTileId(uint instanceIndex) {
  uvec4 v4 = mapTileIds[packedArrayGetU8IndexFromUVEC4Array(instanceIndex)];
  return packedArrayGetU8FromUVEC4ArrayValue(instanceIndex, v4);
}