/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "mapshaderdata.h"
#include "assert/assert.h"
#include "util/memory.h"

shaderbuffer_t MAP_SHADER_DATA_BUFFER;
mapshaderdata_t MAP_SHADER_DATA_DATA;

void mapShaderDataInit() {
  memoryZero(&MAP_SHADER_DATA_DATA, sizeof(mapshaderdata_t));
  shaderBufferInit(&MAP_SHADER_DATA_BUFFER, sizeof(mapshaderdata_t));

  assertTrue(
    sizeof(MAP_SHADER_DATA_DATA.tileIds) == sizeof(OVERWORLD.map.tileIds),
    "Map shader tile data and Overworld tile data are not the same size."
  );
}

void mapShaderDataUpdate() {
  // Copy tile ids.
  memoryCopyRangeSafe(
    MAP_SHADER_DATA_DATA.tileIds,
    OVERWORLD.map.tileIds,
    &OVERWORLD.map.tileIds[
      OVERWORLD.map.width * OVERWORLD.map.height * OVERWORLD.map.layerCount
    ],
    sizeof(MAP_SHADER_DATA_DATA.tileIds)
  );

  // Copy map size.
  memoryCopyRangeSafe(
    &MAP_SHADER_DATA_DATA.mapSize,
    &OVERWORLD.map.width,
    &OVERWORLD.map.layerCount + sizeof(uint8_t),
    sizeof(uint8_t) * 3
  );
  
  shaderBufferBind(&MAP_SHADER_DATA_BUFFER);
  shaderBufferSetData(&MAP_SHADER_DATA_BUFFER, &MAP_SHADER_DATA_DATA);
}

void mapShaderDataDispose() {
  shaderBufferDispose(&MAP_SHADER_DATA_BUFFER);
}