// Copyright (c) 2025 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "../../../../dusk/display/tilesetdefs.h"
#include "../fragments/packed.glsl"

struct Tileset {
  uvec4 tileset;
};

layout(std140) uniform b_Tilesets {
  Tileset tilesets[TILESET_SLOT_COUNT];
};

uniform sampler2D u_TilesetTextures[TILESET_SLOT_COUNT];

uint tilesetGetColumns(uint tilesetIndex) {
  return packedGetU32(0u, tilesets[tilesetIndex].tileset);
}

uint tilesetGetRows(uint tilesetIndex) {
  return packedGetU32(1u, tilesets[tilesetIndex].tileset);
}

uint tilesetGetWidth(uint tilesetIndex) {
  return packedGetU32(2u, tilesets[tilesetIndex].tileset);
}

uint tilesetGetHeight(uint tilesetIndex) {
  return packedGetU32(3u, tilesets[tilesetIndex].tileset);
}

vec4 tilesetGetUVs(uint tilesetIndex, uint col, uint row) {
  Tileset tileset = tilesets[tilesetIndex];
  float segWidth = 1.0 / float(tilesetGetColumns(tilesetIndex));
  float segHeight = 1.0 / float(tilesetGetRows(tilesetIndex));
  float x = float(col) * segWidth;
  float y = float(row) * segHeight;
  return vec4(x, y, x + segWidth, y + segHeight);
}

vec4 tilesetGetUVsByIndex(uint tilesetIndex, uint index) {
  Tileset tileset = tilesets[tilesetIndex];
  uint col = index % tilesetGetColumns(tilesetIndex);
  uint row = index / tilesetGetColumns(tilesetIndex);
  return tilesetGetUVs(tilesetIndex, col, row);
}

vec4 tilesetGetColor(uint tilesetIndex, vec2 coord) {
  switch(tilesetIndex) {
    case 0u:
      return texture(u_TilesetTextures[0], coord);
    case 1u:
      return texture(u_TilesetTextures[1], coord);
    case 2u:
      return texture(u_TilesetTextures[2], coord);
    case 3u:
      return texture(u_TilesetTextures[3], coord);
    default:
      return vec4(1, 1, 1, 1);
  }
}