/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "tilesetshaderdata.h"
#include "assert/assert.h"
#include "util/memory.h"
#include "display/shader/shader.h"

shaderbuffer_t TILESET_SHADER_DATA_BUFFER;
tilesetshaderdata_t TILESET_SHADER_DATA_DATA;
GLuint TILESET_SHADER_DATA_TEXTURES[TILESET_SLOT_COUNT];

void tilesetShaderDataInit() {
  memoryZero(&TILESET_SHADER_DATA_DATA, sizeof(tilesetshaderdata_t));
  shaderBufferInit(&TILESET_SHADER_DATA_BUFFER, sizeof(tilesetshaderdata_t));

  assertTrue(
    sizeof(tilesetshaderdata_t) == sizeof(uvec4_t) * TILESET_SLOT_COUNT,
    "Tileset Shader Data size mismatch"
  );

  uint8_t i = 0;
  do {
    TILESET_SHADER_DATA_TEXTURES[i] = i;
  } while(++i < TILESET_SLOT_COUNT);
}

void tilesetShaderDataUpdate() {
  uint8_t i = 0;
  do {
    tilesetid_t id = TILESET_SLOTS[i];
    if(id == TILESET_NULL) continue;
    tileset_t *tileset = &TILESETS[id];
    texture_t *texture = &TILESET_GL_TEXTURES[i];
    tilesetshaderdatatileset_t *dest = &TILESET_SHADER_DATA_DATA.tilesets[i];

    dest->columns = tileset->columns;
    dest->rows = tileset->rows;
    dest->width = texture->width;
    dest->height = texture->height;
  } while(++i < TILESET_SLOT_COUNT);

  shaderBufferBind(&TILESET_SHADER_DATA_BUFFER);
  shaderBufferSetData(&TILESET_SHADER_DATA_BUFFER, &TILESET_SHADER_DATA_DATA);
}

void tilesetShaderTexturesBind(const GLuint uniform) {
  shaderSetTextures(uniform, TILESET_SHADER_DATA_TEXTURES, TILESET_SLOT_COUNT);
}

void tilesetShaderDataDispose() {
  shaderBufferDispose(&TILESET_SHADER_DATA_BUFFER);
}