// Copyright (c) 2025 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#define PACKED_U8_PER_UI 4
#define PACKED_U8_PER_UVEC4 PACKED_U8_PER_UI * 4

uint packedGetU8(uint position, uvec4 data) {
  uint subData = data[position / 4u];
  return (subData >> (position * 8u)) & 0xFFu;
}

uint packedGetU32(uint position, uvec4 data) {
  return data[position];
}

int packedGetI8(uint position, uvec4 data) {
  uint subData = data[position / 4u];

  int shift = int(position * 8u);
  return int(subData << (24 - shift)) >> 24;
}

uint packedArrayGetU8IndexFromUVEC4Array(uint u8ArrayIndex) {
  // Given a uint8_t array is uploaded, this will return the index to get the
  // appropriate uvec4 from a uvec4 array that will be at the right index.
  return u8ArrayIndex / uint(PACKED_U8_PER_UVEC4);
}

uint packedArrayGetU8FromUVEC4ArrayValue(uint u8ArrayIndex, uvec4 data) {
  // Given a value from a uint8_t array, this will return the value at the
  // appropriate index. You must first get the uvec4 from the array using
  // packedArrayGetU8IndexFromUVEC4Array.
  uint subIndex = (u8ArrayIndex % uint(PACKED_U8_PER_UVEC4)) / uint(PACKED_U8_PER_UI);
  uint shiftAmount = (u8ArrayIndex % uint(PACKED_U8_PER_UI)) * 8u;
  uint value = (data[subIndex] >> shiftAmount) & 0xFFu;
  return value;
}