/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "mapshader.h"
#include "util/memory.h"
#include "display/shader/data/mapshaderdata.h"
#include "display/shader/data/transforms.h"
#include "map_vert.glsl.h"
#include "map_frag.glsl.h"

mapshader_t MAP_SHADER;

void mapShaderInit() {
  memoryZero(&MAP_SHADER, sizeof(mapshader_t));

  shaderInit(
    &MAP_SHADER.shader,
    map_vertShaderSource,
    map_fragShaderSource
  );

  // Uniform blocks
  MAP_SHADER.mapBlock = shaderGetBlock(
    &MAP_SHADER.shader,
    MAP_BLOCK_NAME
  );

  MAP_SHADER.transformsBlock = shaderGetBlock(
    &MAP_SHADER.shader,
    TRANSFORMS_BLOCK_NAME
  );
}

void mapShaderUse() {
  shaderUse(&MAP_SHADER.shader);
  shaderBufferBindToBlock(&MAP_SHADER_DATA_BUFFER, MAP_SHADER.mapBlock);
  shaderBufferBindToBlock(&TRANSFORMS_BUFFER, MAP_SHADER.transformsBlock);
}

void mapShaderDispose() {
  shaderDispose(&MAP_SHADER.shader);
}