/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "assert/assert.h"
#include "shaderbuffer.h"
#include "assert/assertgl.h"

void shaderBufferInit(shaderbuffer_t *shaderBuffer, const size_t size) {
  assertNotNull(shaderBuffer, "shaderBuffer cannot be NULL.");
  assertTrue(size > 0, "size must be greater than 0.");

  shaderBuffer->size = size;

  glGenBuffers(1, &shaderBuffer->id);
  assertNoGLError();

  shaderBufferBind(shaderBuffer);

  glBufferData(GL_UNIFORM_BUFFER, size, NULL, GL_STATIC_DRAW);
}

void shaderBufferBind(shaderbuffer_t *shaderBuffer) {
  assertNotNull(shaderBuffer, "shaderBuffer cannot be NULL.");

  glBindBuffer(GL_UNIFORM_BUFFER, shaderBuffer->id);
  assertNoGLError();
}

void shaderBufferSetData(shaderbuffer_t *shaderBuffer, const void *data) {
  assertNotNull(shaderBuffer, "shaderBuffer cannot be NULL.");
  assertNotNull(data, "data cannot be NULL.");

  glBufferData(GL_UNIFORM_BUFFER, shaderBuffer->size, data, GL_STATIC_DRAW);
  assertNoGLError();
}

void shaderBufferBindToBlock(
  shaderbuffer_t *shaderBuffer,
  const GLuint blockIndex
) {
  assertNotNull(shaderBuffer, "shaderBuffer cannot be NULL.");

  glBindBufferBase(GL_UNIFORM_BUFFER, blockIndex, shaderBuffer->id);
  assertNoGLError();
}

void shaderBufferDispose(shaderbuffer_t *shaderBuffer) {
  assertNotNull(shaderBuffer, "shaderBuffer cannot be NULL.");
  glDeleteBuffers(1, &shaderBuffer->id);
  assertNoGLError();
}