/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "shadermanager.h"
#include "assert/assert.h"
#include "display/shader/data/transforms.h"
#include "display/shader/data/entities.h"
#include "display/shader/data/mapshaderdata.h"
#include "display/shader/data/tilesetshaderdata.h"
#include "display/shader/entityshader/entityshader.h"
#include "display/shader/mapshader/mapshader.h"

shadermanagerdatacallback_t SHADER_MANAGER_DATA_CALLBACKS[] = {
  { transformsInit, transformsUpdate, transformsDispose },
  { entitiesInit, entitiesUpdate, entitiesDispose },
  { mapShaderDataInit, mapShaderDataUpdate, mapShaderDataDispose },
  { tilesetShaderDataInit, tilesetShaderDataUpdate, tilesetShaderDataDispose }
};


shadermanagershadercallback_t SHADER_MANAGER_SHADER_CALLBACKS[] = {
  { entityShaderInit, entityShaderDispose },
  { mapShaderInit, mapShaderDispose }
};

void shaderManagerInit() {
  size_t i;
  assertTrue(
    SHADER_MANAGER_SHADER_CALLBACKS_SIZE > 0,
    "No Shader Callbacks Defined"
  );
  assertTrue(
    SHADER_MANAGER_DATA_CALLBACKS_SIZE > 0,
    "No Data Callbacks Defined"
  );

  // Init Shader Datas (before the shaders)
  i = 0;
  do {
    assertNotNull(
      SHADER_MANAGER_DATA_CALLBACKS[i].init,
      "Data Callback is NULL"
    );
    SHADER_MANAGER_DATA_CALLBACKS[i++].init();
  } while(i < SHADER_MANAGER_DATA_CALLBACKS_SIZE);

  // Init the shaders
  i = 0;
  do {
    assertNotNull(
      SHADER_MANAGER_SHADER_CALLBACKS[i].init,
      "Shader Callback is NULL"
    );
    SHADER_MANAGER_SHADER_CALLBACKS[i++].init();
  } while(i < SHADER_MANAGER_SHADER_CALLBACKS_SIZE);
}

void shaderManagerUpdate() {
  assertTrue(
    SHADER_MANAGER_DATA_CALLBACKS_SIZE > 0,
    "No Data Callbacks Defined"
  );


  // Update all the data
  size_t i = 0;
  do {
    assertNotNull(
      SHADER_MANAGER_DATA_CALLBACKS[i].update,
      "Data Callback is NULL"
    );
    SHADER_MANAGER_DATA_CALLBACKS[i++].update();
  } while(i < SHADER_MANAGER_DATA_CALLBACKS_SIZE);
}

void shaderManagerDispose() {
  assertTrue(
    SHADER_MANAGER_SHADER_CALLBACKS_SIZE > 0,
    "No Shader Callbacks Defined"
  );
  assertTrue(
    SHADER_MANAGER_DATA_CALLBACKS_SIZE > 0,
    "No Data Callbacks Defined"
  );

  // Cleanup the shaders
  size_t i = 0;
  do {
    assertNotNull(
      SHADER_MANAGER_SHADER_CALLBACKS[i].dispose,
      "Shader Callback is NULL"
    );
    SHADER_MANAGER_SHADER_CALLBACKS[i++].dispose();
  } while(i < SHADER_MANAGER_SHADER_CALLBACKS_SIZE);

  // Cleanup the data
  i = 0;
  do {
    assertNotNull(
      SHADER_MANAGER_DATA_CALLBACKS[i].dispose,
      "Data Callback is NULL"
    );
    SHADER_MANAGER_DATA_CALLBACKS[i++].dispose();
  } while(i < SHADER_MANAGER_DATA_CALLBACKS_SIZE);
}