/**
 * Copyright (c) 2023 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "duskgl.h"
#include <spng.h>

extern int32_t TEXTURE_ACTIVE_COUNT;

typedef struct {
  uint32_t width;
  uint32_t height;
  GLuint id;
} texture_t;

/**
 * Initializes a texture to be read to be used.
 * 
 * @param texture Texture to initialize.
 * @param path Path of the file to load.
 */
void textureLoad(
  texture_t *texture,
  const char_t *path
);

/**
 * Binds the texture to the given slot (for use by the shaders).
 * 
 * @param texture Texture to bind.
 * @param slot Slot to bind to.
 */
void textureBind(
  const texture_t *texture,
  const GLuint slot
);

/**
 * Unloads a previously initialized texture.
 * 
 * @param texture Texture to destroy.
 */
void textureDispose(texture_t *texture);

/**
 * Callback function for libspng to read from the asset.
 * 
 * @param ctx The spng context.
 * @param user User data.
 * @param destination Destination buffer.
 * @param length Length of the buffer.
 * @return The amount of data read.
 */
int32_t textureSPNGBuffer(
  spng_ctx *ctx,
  void *user,
  void *destination,
  size_t length
);