/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "input.h"
#include "display/window.h"

typedef struct {
  uint16_t glfw;
  inputstate_t bind;
} glfwkeybindmap_t;

glfwkeybindmap_t GLFW_KEY_BIND_MAP[] = {
  {GLFW_KEY_W, INPUT_UP},
  {GLFW_KEY_UP, INPUT_UP},
  
  {GLFW_KEY_S, INPUT_DOWN},
  {GLFW_KEY_DOWN, INPUT_DOWN},
  
  {GLFW_KEY_A, INPUT_LEFT},
  {GLFW_KEY_LEFT, INPUT_LEFT},
  
  {GLFW_KEY_D, INPUT_RIGHT},
  {GLFW_KEY_RIGHT, INPUT_RIGHT},
  
  {GLFW_KEY_ENTER, INPUT_ACCEPT},
  {GLFW_KEY_E, INPUT_ACCEPT},
  {GLFW_KEY_SPACE, INPUT_ACCEPT},

  {GLFW_KEY_BACKSPACE, INPUT_BACK},
  {GLFW_KEY_Q, INPUT_BACK},
  
  {GLFW_KEY_ESCAPE, INPUT_MENU},

  {0, 0}
};

inputstate_t inputPlatformState() {
  inputstate_t state = 0;

  // Handle keybinds
  glfwkeybindmap_t *map = GLFW_KEY_BIND_MAP;
  do {
    if(glfwGetKey(window, map->glfw) != GLFW_PRESS) {
      map++;
      continue;
    }
    state |= map->bind;
    map++;
  } while(map->glfw != 0);

  return state;
}